@@ -21,7 +21,7 @@
 
 SEXP do_aquarootscript do_formals
 {
-    do_start("aquarootscript", 0);
+    do_start_no_op_rho("aquarootscript", 0);
 
 
     errorcall(call, "not implemented yet");
