@@ -7,7 +7,7 @@ main <- function ()
             invisible(loadNamespace("microbenchmark"))
             invisible(loadNamespace("this.path"))
             print(this.path:::.faster.subsequent.times.test())
-        } else cat("\npackage {microbenchmark} is not available :(\n")
+        } else cat("\n'package:microbenchmark' is not available :(\n")
     })
     cat("\n")
     this.path:::.Rscript(c("--default-packages=NULL", "--vanilla", FILE))
